package nl.uvt.locator;

import java.io.File;
import java.util.Properties;

import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Nodes;
import nu.xom.xslt.XSLTransform;


public class AvailabilityConnection {

	Document query( String epn ) throws LocatorException {
		Document result = null;
		String url = Locator.locatorConfig.getProperty("availabilityUri");
		if ( url.contains("pica3") ){
			// old style using pica3 requests to an mps 
			try {
				Properties prop = new Properties();
				prop.setProperty("action", "listVolumes" );
				prop.setProperty("epn", epn );
				url = HttpConnection.createUrlString(url, prop );
				Locator.logger.debug("Try URL: " + url );
				Document res = HttpConnection.httpGetDocument( url, Locator.httpConfig );
//				Locator.logger.debug( "availability doc:" + res.toXML());
				int hits = extractHits( res );
				Locator.logger.debug("availability numOfItems=" + hits );
				if ( hits > 0 ){
					String sheetName = Locator.locatorConfig.getProperty("availabilitySheet");
					result = getAvailability( res, sheetName );
				}				
			}
			catch ( Exception e) {
				throw new LocatorException( "Availability URL: '" + url + "' failed"); 
			}
		}
		else {
			// new style using getAvailability requests to an mps 
			try {
				Properties prop = new Properties();
				prop.setProperty("id", epn );
				prop.setProperty("id_type", "bib" );
				url = HttpConnection.createUrlString(url, prop );
				Locator.logger.debug("Try URL: " + url );
				result = HttpConnection.httpGetDocument( url, Locator.httpConfig );
//				Locator.logger.debug( "availability doc:" + result.toXML());
			}				
			catch ( Exception e) {
				throw new LocatorException( "Availability URL: '" + url + "' failed"); 
			}
		}
		return result;
	}

	static public int extractHits( Document doc ){
		Nodes numOfRecords = doc.query("//numberOfItems"); 
		if( numOfRecords.size() > 0 ){
			Element numHitsElement = (Element) numOfRecords.get(0);
			String hitString = numHitsElement.getValue();
			int hits=Integer.parseInt(hitString);
			return hits;
		}
		return 0;
	}

	static public Document getAvailability( Document doc, String sheetName ){
		Builder builder = new Builder();
		try {
			Locator.logger.debug( "use xslt sheet:" + sheetName );
			File f = Locator.getFile( sheetName );
			if ( f != null ){
				nu.xom.Document stylesheet = builder.build(f);
				XSLTransform transform = new XSLTransform(stylesheet);
//				Locator.logger.debug( "input doc:" + doc.toXML());
				Nodes output = transform.transform( doc );
				nu.xom.Document standardDoc = XSLTransform.toDocument(output);
//				Locator.logger.debug( "transformed doc:" + standardDoc.toXML());
				return standardDoc;
			}
			else {
				Locator.logger.fatal("unable to get a stylesheet:" + sheetName );
			}
		}
		catch( Exception e )
		{
			Locator.logger.error( "XSLT processor exception", e );
		}
		return null;
	}


}
