package nl.uvt.locator;

import java.util.List;
import java.util.Properties;

public class MerescoConnection {

	List<LinkInfo> query( QueryData pQ ) throws LocatorException {
		List<LinkInfo> meResult = null;
		nu.xom.Document result;
		String url = Locator.locatorConfig.getProperty("merescoUri");
		String query = "";
		try {
			Properties prop = new Properties( Locator.sruDefaultProps );
			prop.setProperty("recordSchema","mods" );
			if ( pQ.epn.length() > 0 ){
				// Thomas would like a direct access to availability here
				// but don't we need stuff like collection, and callNumber?
				Locator.logger.debug("direct search with EPN: " + pQ.epn );
				query = "mods.location.holdingSimple.copyInformation.note=" + pQ.epn;						
			}	
			else if ( pQ.genre.equals( "article") ){
				if ( pQ.issn.length() > 0 ){
					query = "mods.identifier=" + pQ.issn;						
				}
//				else if ( pQ.location.length() > 0 ){
//					String location = fixLocation( pQ.location );
//					query = "mods.location.holdingSimple.copyInformation.shelfLocator=\"" + location + "\"";
//				}
				else if ( pQ.title.length() > 0 ) {
					// titles like "bla di bla: argh" are assumed to be "bla di bla" with subtitle "argh"
					String title = pQ.title;
					String titleParts[] = title.split( ":", 2 );
					if ( titleParts.length > 1 ){
						for( int i=0; i < titleParts.length; ++i )
							Locator.logger.debug( "part["+i+"]= '"+titleParts[i] +"'");
								title = titleParts[0];
					}
					query = "mods.titleInfo.title=\"" + title + "\"";
				}
			}
			else if ( pQ.isbn.length() > 0 ){
					Locator.logger.debug("ISBN: " + pQ.isbn );
					query = "mods.identifier=" + pQ.isbn;						
				}
			else if ( pQ.location.length() > 0 ){
				String location = fixLocation( pQ.location );
				query = "mods.location.holdingSimple.copyInformation.shelfLocator=\"" + location + "\"";
			}
			else {
				String title = null;
				if ( pQ.atitle.length() > 0 ) {
					title = pQ.atitle;
				}
				else if ( pQ.title.length() > 0 ) {
					title = pQ.title;
				}	
				String author = null;
				if ( !pQ.genre.equals("bookitem") && 	
						!pQ.genre.equals("proceeding") &&
						pQ.aulast.length() > 0 ){
					author = pQ.aulast;
				}
				if ( title != null ) {
					query = "mods.titleInfo.title=\"" + title + "\"";
					if ( author != null )
						query += " and " + "mods.titleInfo.author=\"" + author + "\"";
				}	
				else if ( author != null )
					query = "mods.titleInfo.author=\"" + author + "\"";	
			}
			if ( query.length() == 0 ){
					// fails when no usefull data available
				return meResult;
			}
			query += " and meta.repository.repositoryGroupId exact opc-uvt-nl";
			prop.setProperty("query", query );
			url = HttpConnection.createUrlString(url, prop );
			Locator.logger.debug("Try URL: " + url );
			result = HttpConnection.httpGetDocument( url, Locator.httpConfig );
			int hits = Locator.extractHits( result );
			Locator.logger.debug("meresco numOfRecords=" + hits );
			if ( hits > 0 ){
//				Locator.logger.debug( "meresco doc:" + result.toXML());
				String sheetName = Locator.locatorConfig.getProperty("merescoSheet");
				meResult = HoldingInfo.getModsResults( result, pQ, sheetName );
			}
		}
		catch ( Exception e) {
			throw new LocatorException( "meresco URL: '" + url + "' failed"); 
		}
		return meResult;
	}

	private String fixLocation( String loc ) {
		String result = loc.replaceAll("/\\d{4}.*", "/...");
		if ( !loc.equals(result))
			Locator.logger.debug("fixLocation replaced '" + loc + "' by: '" + result + "'" );			
		return result;
	}

}
